/**
 * A tiny and mostly spec-compliant WebWorker polyfill
 *
 * This WebWorker polyfill is implemented on top of an `iframe` element to allow
 * web workers in contexts when `XMLHTTPRequest` is not available or blocked by
 * the browser, e.g. when using the `file:` protocol. Note that this polyfill
 * cannot provide asynchronous workers, as iframes are synchronous by nature.
 */
export declare class IFrameWorker implements Worker {
    protected url: string | URL;
    addEventListener: Worker["addEventListener"];
    removeEventListener: Worker["removeEventListener"];
    dispatchEvent: Worker["dispatchEvent"];
    onerror: Worker["onerror"];
    onmessage: Worker["onmessage"];
    onmessageerror: Worker["onmessageerror"];
    /**
     * The `iframe` we'll use to simulate a worker
     */
    iframe: HTMLIFrameElement;
    /**
     * A promise indicating whether the `iframe` was initialized
     */
    protected ready: Promise<unknown>;
    /**
     * Create a new worker from the given URL
     *
     * @param url - Worker script URL
     * @param options - Worker options
     */
    constructor(url: string | URL, options?: WorkerOptions);
    /**
     * Immediately terminate the worker
     */
    terminate(): void;
    /**
     * Send a message to the worker
     *
     * @param data - Message data
     */
    postMessage(data: unknown): void;
    /**
     * Helper to retrieve the content window of the `iframe`
     *
     * If the window doesn't exist, e.g. when the `iframe` has been removed by
     * terminating the web worker, an error is thrown.
     *
     * @returns Window
     */
    protected get worker(): Window;
    /**
     * Internal listener that proxies message events
     *
     * @param ev - Message event
     */
    protected handleMessage: (ev: MessageEvent) => void;
    /**
     * Internal listener that proxies error events
     *
     * @param message - Error message
     * @param filename - Source filename
     * @param lineno - Source line number
     * @param colno - Source column number
     * @param error - Error object
     */
    protected handleError: (message?: string | undefined, filename?: string | undefined, lineno?: number | undefined, colno?: number | undefined, error?: Error | undefined) => void;
}
//# sourceMappingURL=index.d.ts.map